@@ -16,7 +16,6 @@
 #include <limits>
 
 #ifdef RNIFTYREG
-#include <R.h>
 #include "substitutions.h"
 #endif
 
@@ -237,21 +236,13 @@ void initialise_block_matching_method(  nifti_image * target,
         case NIFTI_TYPE_FLOAT64:
             _reg_set_active_blocks<double>(target, params, mask, runningOnGPU);break;
         default:
-#ifdef RNIFTYREG
-            error("The target image data type is not supported");
-#else
             fprintf(stderr,"[NiftyReg ERROR] initialise_block_matching_method\tThe target image data type is not supported\n");
             exit(1);
-#endif
     }
     if(params->activeBlockNumber<2){
-#ifdef RNIFTYREG
-        error("There are no active blocks");
-#else
         fprintf(stderr,"[NiftyReg ERROR] There are no active blocks\n");
         fprintf(stderr,"[NiftyReg ERROR] ... Exit ...\n");
         exit(1);
-#endif
     }
 #ifndef NDEBUG
     printf("[NiftyReg DEBUG]: There are %i active block(s) out of %i.\n", params->activeBlockNumber, params->blockNumber[0]*params->blockNumber[1]*params->blockNumber[2]);
@@ -1767,267 +1758,3 @@ void optimize(	_reg_blockMatchingParam *params,
         }
     }
 }
-
-// Routines for alculating Singular Value Decomposition follows.
-// Adopted from Numerical Recipes in C.
-
-#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
-
-static float maxarg1,maxarg2;
-#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
-(maxarg1) : (maxarg2))
-
-static int iminarg1,iminarg2;
-#define IMIN(a,b) (iminarg1=(a),iminarg2=(b),(iminarg1) < (iminarg2) ?\
-(iminarg1) : (iminarg2))
-
-static float sqrarg;
-#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)
-
-// Calculate pythagorean distance
-float pythag(float a, float b)
-{
-    float absa, absb;
-    absa = fabs(a);
-    absb = fabs(b);
-
-    if (absa > absb) return (float)(absa * sqrt(1.0f+SQR(absb/absa)));
-    else return (absb == 0.0f ? 0.0f : (float)(absb * sqrt(1.0f+SQR(absa/absb))));
-}
-
-void svd(float ** in, int m, int n, float * w, float ** v)
-{
-    float * rv1 = (float *)malloc(sizeof(float) * n);
-    float anorm, c, f, g, h, s, scale, x, y, z;
-    int flag,i,its,j,jj,k,l=0,nm=0;
-
-    g = scale = anorm = 0.0f;
-    for (i = 1; i <= n; ++i)
-    {
-        l = i + 1;
-        rv1[i-1] = scale * g;
-        g = s = scale = 0.0f;
-
-        if ( i <= m)
-        {
-            for (k = i; k <= m; ++k)
-            {
-                scale += fabs(in[k-1][i-1]);
-            }
-            if (scale)
-            {
-                for (k = i; k <= m; ++k)
-                {
-                    in[k-1][i-1] /= scale;
-                    s += in[k-1][i-1] * in[k-1][i-1];
-                }
-                f = in[i-1][i-1];
-                g = -SIGN(sqrt(s), f);
-                h = f * g - s;
-                in[i-1][i-1] = f - g;
-
-                for (j = l; j <= n; ++j)
-                {
-                    for (s = 0.0, k=i; k<=m; ++k) s += in[k-1][i-1]*in[k-1][j-1];
-                    f = s/h;
-                    for (k = i; k <= m; ++k) in[k-1][j-1] += f * in[k-1][i-1];
-                }
-                for (k = i; k <= m; ++k)
-                {
-                    in[k-1][i-1] *= scale;
-                }
-            }
-        }
-        w[i-1] = scale * g;
-        g = s = scale = 0.0;
-        if ((i <= m) && (i != n))
-        {
-            for (k = l; k <= n; ++k)
-            {
-                scale += fabs(in[i-1][k-1]);
-            }
-            if (scale)
-            {
-                for (k = l; k <= n; ++k)
-                {
-                    in[i-1][k-1] /= scale;
-                    s += in[i-1][k-1] * in[i-1][k-1];
-                }
-                f = in[i-1][l-1];
-                g = -SIGN(sqrt(s), f);
-                h = f*g-s;
-                in[i-1][l-1] = f - g;
-
-                for (k = l; k <= n; ++k) rv1[k-1] = in[i-1][k-1]/h;
-                for (j = l; j <= m; ++j)
-                {
-                    for (s = 0.0, k = l; k <= n; ++k)
-                    {
-                        s += in[j-1][k-1] * in[i-1][k-1];
-                    }
-                    for (k = l; k <= n; ++k)
-                    {
-                        in[j-1][k-1] += s * rv1[k-1];
-                    }
-                }
-
-                for (k=l;k<=n;++k) in[i-1][k-1] *= scale;
-            }
-        }
-        anorm = FMAX(anorm, (fabs(w[i-1])+fabs(rv1[i-1])));
-    }
-
-    for (i = n; i >= 1; --i)
-    {
-        if (i < n)
-        {
-            if (g)
-            {
-                for (j = l; j <= n; ++j)
-                {
-                    v[j-1][i-1] = (in[i-1][j-1]/in[i-1][l-1])/g;
-                }
-                for (j = l; j <= n; ++j)
-                {
-                    for (s = 0.0, k = l; k <= n; ++k) s += in[i-1][k-1] * v[k-1][j-1];
-                    for (k=l;k<=n;++k) v[k-1][j-1] += s * v[k-1][i-1];
-                }
-            }
-            for (j = l; j <= n; ++j) v[i-1][j-1] = v[j-1][i-1] = 0.0;
-        }
-        v[i-1][i-1] = 1.0f;
-        g = rv1[i-1];
-        l = i;
-    }
-
-    for (i = IMIN(m, n); i >= 1; --i)
-    {
-        l = i+1;
-        g = w[i-1];
-        for (j = l; j <= n; ++j) in[i-1][j-1] = 0.0f;
-        if (g)
-        {
-            g = 1.0f/g;
-            for (j = l; j <= n; ++j)
-            {
-                for (s = 0.0, k = l; k <= m; ++k) s += in[k-1][i-1] * in[k-1][j-1];
-                f = (s/in[i-1][i-1])*g;
-                for (k=i; k <=m; ++k) in[k-1][j-1] += f * in[k-1][i-1];
-            }
-            for (j=i; j <= m; ++j) in[j-1][i-1] *= g;
-        }
-        else for (j = i; j <= m; ++j) in[j-1][i-1] = 0.0;
-        ++in[i-1][i-1];
-    }
-
-    for (k = n; k >= 1; --k)
-    {
-        for (its = 0; its < 30; ++its)
-        {
-            flag = 1;
-            for (l=k; l >= 1; --l)
-            {
-                nm = l - 1;
-                if ((float)(fabs(rv1[l-1])+anorm) == anorm)
-                {
-                    flag = 0;
-                    break;
-                }
-                if ((float)(fabs(w[nm-1])+anorm) == anorm) break;
-            }
-
-            if (flag)
-            {
-                c = 0.0f;
-                s = 1.0f;
-                for (i=l; i<=k; ++i) // changed
-                {
-                    f = s * rv1[i-1];
-                    rv1[i-1] = c * rv1[i-1];
-                    if ((float)(fabs(f)+anorm) == anorm) break;
-                    g=w[i-1];
-                    h=pythag(f,g);
-                    w[i-1]=h;
-                    h=1.0f/h;
-                    c=g*h;
-                    s = -f*h;
-
-                    for (j = 1; j <= m; ++j)
-                    {
-                        y=in[j-1][nm-1];
-                        z=in[j-1][i-1];
-                        in[j-1][nm-1]=y*c+z*s;
-                        in[j-1][i-1]=z*c-y*s;
-                    }
-                }
-            }
-            z = w[k-1];
-            if (l == k)
-            {
-                if (z < 0.0f)
-                {
-                    w[k-1] = -z;
-                    for (j = 1; j <= n; ++j) v[j-1][k-1] = -v[j-1][k-1];
-                }
-                break;
-            }
-
-            x = w[l-1];
-            nm = k - 1;
-            y = w[nm-1];
-            g = rv1[nm-1];
-            h = rv1[k-1];
-
-            f = ((y-z)*(y+z)+(g-h)*(g+h))/(2.0f*h*y);
-            g = pythag(f, 1.0f);
-            f = ((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
-            c = s = 1.0f;
-            for (j = l; j <= nm; ++j)
-            {
-                i = j + 1;
-                g = rv1[i-1];
-                y = w[i-1];
-                h = s * g;
-                g = c * g;
-                z = pythag(f, h);
-                rv1[j-1] = z;
-                c = f/z;
-                s = h/z;
-                f = x*c+g*s;
-                g = g*c-x*s;
-                h = y*s;
-                y *= c;
-
-                for (jj = 1; jj <= n; ++jj)
-                {
-                    x = v[jj-1][j-1];
-                    z = v[jj-1][i-1];
-                    v[jj-1][j-1] = x*c+z*s;
-                    v[jj-1][i-1] = z*c-x*s;
-                }
-                z = pythag(f, h);
-                w[j-1] = z;
-                if (z)
-                {
-                    z = 1.0f/z;
-                    c = f * z;
-                    s = h * z;
-                }
-                f = c*g+s*y;
-                x = c*y-s*g;
-
-                for (jj = 1; jj <= m; ++jj)
-                {
-                    y = in[jj-1][j-1];
-                    z = in[jj-1][i-1];
-                    in[jj-1][j-1] = y*c+z*s;
-                    in[jj-1][i-1] = z*c-y*s;
-                }
-            }
-            rv1[l-1] = 0.0f;
-            rv1[k-1] = f;
-            w[k-1] = x;
-        }
-    }
-    free (rv1);
-}
